﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2017 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HEAL.Attic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.PermutationEncoding;

namespace HeuristicLab.Analysis.FitnessLandscape {
  [Item("QAP Path-Relinking Problem Instance Analyzer", "")]
  [StorableType("9A603F8E-8BA9-4FF4-9694-14ED01E68657")]
  public sealed class QAPPRProblemInstanceAnalyzer : PrProblemInstanceAnalyzer<Permutation> {

    [StorableConstructor]
    private QAPPRProblemInstanceAnalyzer(StorableConstructorFlag _) : base(_) { }
    private QAPPRProblemInstanceAnalyzer(QAPPRProblemInstanceAnalyzer original, Cloner cloner) : base(original, cloner) { }
    public QAPPRProblemInstanceAnalyzer() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new QAPPRProblemInstanceAnalyzer(this, cloner);
    }

    protected override DoubleArray GetCharacteristics() {
      var paths = PathsParameter.ActualValue;
      if (paths == null || paths.IsEmpty) return null;
      var trajectories = paths.Paths.ToList();
      var length = trajectories[0].First().Item1.Length;
      trajectories = paths.Paths.Where(x => x.Count > 0.75 * length).ToList();
      if (trajectories.Count == 0) return null;

      return new DoubleArray(PermutationPathAnalysis.GetCharacteristics(trajectories).GetValues().ToArray());
    }
  }
}
